require 'sink'
require 'thread'
require 'drb/drb'

module Sink
  class Command
    def initialize(dot, argv)
      @dot = dot
      @manager = Manager.new(dot)
      @cmd = (argv.shift || "status").intern
      @uri = argv.shift || ENV['SINKROOT']
    end

    def method_missing(msg_id, *a, &b)
      raise("unknown sub-command")
    end
    
    def perform
      self.__send__(@cmd)
    end

    def status
      @manager.report do |sts|
	print sts.desc
      end
    end

    def checkin
      open_sinkroot do |root|
	@manager.sync(root)
	root.sync(@manager)
      end
    end
    alias ci checkin
    alias in checkin

    def checkout
      open_sinkroot do |root|
	root.sync(@manager)
	@manager.sync(root)
      end
    end
    alias co checkout
    alias out checkin

    def debug
      p @manager
    end

    private
    def open_sinkroot
      raise('$SINKROOT: nil') unless @uri
      DRb.start_service
      front = DRbObject.new(nil, @uri)
      front.open do |there|
	yield(there)
      end
    end
  end
end

if __FILE__ == $0
  Sink::DotSink.transaction do |dot|
    begin
      Sink::Command.new(dot, ARGV).perform
      dot.commit
    rescue
      p $!
      puts $@
      dot.abort
    end
  end
end

